@@ -402,7 +402,7 @@
 #define DEFAULT_NO_CONTROL_LOG 0
 #define DEFAULT_NUM_OF_WHOWAS 1
 #define DEFAULT_PAD_CHAR ' '
-#define DEFAULT_QUIT_MESSAGE "ircII %s -- Accept no limitations"
+#define DEFAULT_QUIT_MESSAGE "ircII %s -- Are we there yet?"
 #define DEFAULT_RANDOM_LOCAL_PORTS 0
 #define DEFAULT_RANDOM_SOURCE 0
 #define DEFAULT_REVERSE_STATUS_LINE 1
