@@ -34,6 +34,7 @@ typedef	struct
 	int	nickname_pending;	/* Is a NICK command pending? */
 	int	resetting_nickname;	/* Is a nickname reset in progress? */
 	int	registration_pending;	/* Is a registration going on ? */
+	int	rejoined_channels;	/* Has we tried to auto-rejoin? */
 	char	*userhost;		/* my userhost on this server */
 	char	*away;			/* away message for this server */
 	int	oper;			/* true if operator */
@@ -182,4 +183,7 @@ const	char *	get_server_redirect		(int);
 	int	check_server_redirect		(const char *);
 	void	save_servers			(FILE *);
 
+	void	server_did_rejoin_channels	(int);
+	int	did_server_rejoin_channels	(int);
+
 #endif /* _SERVER_H_ */
