@@ -69,6 +69,7 @@ enum VAR_TYPES {
 	HIGHLIGHT_CHAR_VAR,
 	HIGH_BIT_ESCAPE_VAR,
 	HISTORY_VAR,
+	HISTORY_CIRCLEQ_VAR,
 	HOLD_INTERVAL_VAR,
 	HOLD_MODE_VAR,
 	INDENT_VAR,
@@ -190,6 +191,7 @@ enum VAR_TYPES {
 	STATUS_WINDOW_VAR,
 	SUPPRESS_FROM_REMOTE_SERVER,
 	SUPPRESS_SERVER_MOTD_VAR,
+	SWITCH_CHANNEL_ON_PART_VAR,
 	TAB_VAR,
 	TAB_MAX_VAR,
 	TERM_DOES_BRIGHT_BLINK_VAR,
