@@ -1165,7 +1165,8 @@ BUILT_IN_COMMAND(funny_stuff)
 		if (!(stuff = get_channel_by_refnum(0)))
 			stuff = empty_string;
 
-	if (strchr(stuff, '*'))
+	/* Channel names can contain stars! */
+	if (strchr(stuff, '*') && !im_on_channel(stuff, from_server))
 	{
 		funny_match(stuff);
 		if (min && ircu)
