@@ -55,6 +55,7 @@ typedef struct in_addr		IA;
 typedef	struct	DCC_struct
 {
 		unsigned	flags;
+		int		locked;		/* XXX - Sigh */
 		int		read;
 		int		write;
 		int		file;
@@ -165,7 +166,19 @@ struct	deadlist
 	struct deadlist *next;
 }	*deadlist = NULL;
 
-int	dcc_dead (void) { return deadlist ? 1 : 0; }
+int	dcc_dead (void) 
+{ 
+	struct deadlist *dies;
+
+	if (!deadlist)
+		return 0;
+
+	for (dies = deadlist; dies; dies = dies->next)
+		if (dies->it->locked)
+			return 0;		/* Erase list is locked */
+
+	return 1;		/* All clear! */
+}
 
 /*
  * remove_from_dcc_list: What do you think it does?
@@ -216,6 +229,12 @@ static void 	dcc_really_erase (void)
 {
 	struct deadlist *dies;
 
+	/* Quick and dirty hack to avoid _NiC's bug */
+	/* XXX -- Emphasis on DIRTY and HACK. */
+	for (dies = deadlist; dies; dies = dies->next)
+		if (dies->it->locked)
+			return;		/* Erase list is locked */
+
 	while ((dies = deadlist))
 	{
 		deadlist = deadlist->next;
@@ -473,6 +492,7 @@ static	DCC_list *dcc_searchlist (
 
 	new_client 			= new_malloc(sizeof(DCC_list));
 	new_client->flags 		= type;
+	new_client->locked		= 0;
 	new_client->read 		= -1;
 	new_client->write 		= -1;
 	new_client->file 		= -1;
@@ -577,6 +597,7 @@ static	int		dcc_open (DCC_list *dcc)
 		 * If this was a two-peer connection, then tell the user
 		 * that the connection was successfull.
 		 */
+		dcc->locked++;
 		if ((dcc->flags & DCC_TYPES) != DCC_RAW)
 		{
                         /* 
@@ -627,6 +648,7 @@ static	int		dcc_open (DCC_list *dcc)
 			    message_from(NULL, LOG_CURRENT);
 			}
 		}
+		dcc->locked--;
 
 		/*
 		 * Record the time the connection was started, 
@@ -739,6 +761,7 @@ static void	dcc_send_booster_ctcp (DCC_list *dcc)
 	/*
 	 * If this is a DCC SEND...
 	 */
+	dcc->locked++;
 	if (dcc->flags & DCC_FILEOFFER)
 	{
 		char *	url_name = dcc_urlencode(nopath);
@@ -808,6 +831,7 @@ static void	dcc_send_booster_ctcp (DCC_list *dcc)
 				type, dcc->user);
 		message_from(NULL, LOG_CURRENT);
 	}
+	dcc->locked--;
 }
 
 
@@ -904,9 +928,11 @@ const	char 		*text_display, 	/* What to tell the user we sent */
 
 	if (noisy)
 	{
+		dcc->locked++;
 		if (do_hook(list, "%s %s", dcc->user, text_display))
 			put_it("=> %c%s%c %s", 
 				thing, dcc->user, thing, text_display);
+		dcc->locked--;
 	}
 
 	message_from(NULL, LOG_CURRENT);
@@ -1092,6 +1118,7 @@ static	void 	dcc_close (char *args)
 		count++;
 		dcc->flags |= DCC_DELETE;
 
+		dcc->locked++;
                 if (do_hook(DCC_LOST_LIST,"%s %s %s USER ABORTED CONNECTION",
 			dcc->user, 
 			dcc_types[my_type],
@@ -1100,6 +1127,7 @@ static	void 	dcc_close (char *args)
 			dcc_types[my_type],
 			file ? file : "<any>", 
 			dcc->user);
+		dcc->locked--;
 	}
 
 	if (!count)
@@ -1266,6 +1294,7 @@ static	char		*format =
 	{
 	    unsigned	flags = Client->flags;
 
+	    Client->locked++;
 	    if (do_hook(DCC_LIST_LIST, "%s %s %s %s %ld %ld %ld %s",
 				dcc_types[flags & DCC_TYPES],
 				Client->encrypt ? one : zero,
@@ -1378,6 +1407,7 @@ static	char		*format =
 						  "Unknown",
 			time_f, size, completed, speed, filename);
 	    }
+	    Client->locked--;
 	}
 	do_hook(DCC_LIST_LIST, "End * * * * * * *");
 }
@@ -1690,11 +1720,13 @@ char	*dcc_raw_connect(char *host, u_short port)
 	}
 
 	Client->user = m_strdup(ltoa(Client->read));
+	Client->locked++;
 	if (do_hook(DCC_RAW_LIST, "%s %s E %d", Client->user, host, port))
             if (do_hook(DCC_CONNECT_LIST,"%s RAW %s %d", 
 				Client->user, host, port))
 		say("DCC RAW connection to %s on %s via %d established", 
 				host, Client->user, port);
+	Client->locked--;
 
 	message_from(NULL, LOG_CURRENT);
 	return m_strdup(Client->user);
@@ -1879,96 +1911,98 @@ void	register_dcc_offer (char *user, char *type, char *description, char *addres
 
   	if (do_auto)
   	{
-                if (do_hook(DCC_CONNECT_LIST,"%s CHAT",user))
+                if (do_hook(DCC_CONNECT_LIST,"%s CHAT", user))
 			say("DCC CHAT already requested by %s, connecting...", user);
   		dcc_chat(user);
+		return;
 	}
+
+	/* 
+	 * DCC SEND and CHAT have different arguments, so they can't
+	 * very well use the exact same hooked data.  Both now are
+	 * identical for $0-4, and SEND adds filename/size in $5-6 
+	 */
+	Client->locked++;
+	if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
+	       jvs_blah = do_hook(DCC_REQUEST_LIST,
+			 "%s %s %s %s %d %s %ld",
+			  user, type, description,
+			  inet_ntoa(Client->remote),
+			  ntohs(Client->remport),
+			  Client->description,
+			  Client->filesize);
 	else
+	       jvs_blah = do_hook(DCC_REQUEST_LIST,
+			 "%s %s %s %s %d",
+			  user, type, description,
+			  inet_ntoa(Client->remote),
+			  ntohs(Client->remport));
+	Client->locked--;
+
+	if (jvs_blah)
 	{
-		/* 
-		 * DCC SEND and CHAT have different arguments, so they can't
-		 * very well use the exact same hooked data.  Both now are
-		 * identical for $0-4, and SEND adds filename/size in $5-6 
-		 */
-		if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
-                       jvs_blah = do_hook(DCC_REQUEST_LIST,
-                                 "%s %s %s %s %d %s %ld",
-                                  user, type, description,
-                                  inet_ntoa(Client->remote),
-                                  ntohs(Client->remport),
-                                  Client->description,
-                                  Client->filesize);
-               else
-                       jvs_blah = do_hook(DCC_REQUEST_LIST,
-                                 "%s %s %s %s %d",
-                                  user, type, description,
-                                  inet_ntoa(Client->remote),
-                                  ntohs(Client->remport));
-
-            /* indentation broken 22 Nov 96, crowman... eeeeyuck */
-	    if ( jvs_blah )
+	    /* These should be more helpful messages now. */
+	    /* WC asked to have them moved here */
+	    if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
 	    {
-		/* These should be more helpful messages now. */
-		/* WC asked to have them moved here */
-		if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
+		struct stat statit;
+
+		if (stat(Client->description, &statit) == 0)
 		{
-			struct stat statit;
-			if (stat(Client->description, &statit) == 0)
-			{
-				if (statit.st_size < Client->filesize)
-				{
-					say("WARNING: File [%s] exists but is smaller then the file offered.", Client->description);
-					say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
+		    if (statit.st_size < Client->filesize)
+		    {
+			say("WARNING: File [%s] exists but is smaller then the file offered.", Client->description);
+			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
 #ifdef MIRC_BROKEN_DCC_RESUME
-					say("Use /DCC RESUME %s %s           to continue the copy where it left off.", user, Client->description);
+			say("Use /DCC RESUME %s %s           to continue the copy where it left off.", user, Client->description);
 #endif
-					say("Use /DCC RENAME %s %s newname   to save it to a different filename", user, Client->description);
-					say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
-				}
-				else if (statit.st_size == Client->filesize)
-				{
-					say("WARNING: File [%s] exists, and its the same size.", Client->description);
+			say("Use /DCC RENAME %s %s newname   to save it to a different filename", user, Client->description);
+			say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
+		    }
+		    else if (statit.st_size == Client->filesize)
+		    {
+			say("WARNING: File [%s] exists, and its the same size.", Client->description);
 #ifdef USE_DCC_CHECKSUM
-					if (Client->cksum)
-					{
-						char buffer[10];
-						char foo[5];
-						file_checksum(foo, Client->description);
-						sprintf(buffer, "%x%x%x%x", foo[0], foo[1], foo[2], foo[3]);
-						if (!strcmp(Client->cksum, buffer))
-							say("WARNING: The two files are confirmed to be identical.");
-					}
+			if (Client->cksum)
+			{
+			    char buffer[10];
+			    char foo[5];
+			    file_checksum(foo, Client->description);
+			    sprintf(buffer, "%x%x%x%x", 
+				foo[0], foo[1], foo[2], foo[3]);
+			    if (!strcmp(Client->cksum, buffer))
+				say("WARNING: The two files are confirmed to be identical.");
+			}
 #endif
 
-					say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
-					say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
-					say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
-				}
-				else 	/* Bigger than */
-				{
-					say("WARNING: File [%s] already exists.", Client->description);
-					say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
-					say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
-					say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
-				}
-			}
+			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
+			say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
+			say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
+		    }
+		    else 	/* Bigger than */
+		    {
+			say("WARNING: File [%s] already exists.", Client->description);
+			say("Use /DCC CLOSE GET %s %s        to not get the file.", user, Client->description);
+			say("Use /DCC RENAME %s %s           to get the file anyways under a different name.", user, Client->description);
+			say("Use /DCC GET %s %s              to overwrite the existing file.", user, Client->description);
+		    }
 		}
+	    }
 
-		/* Thanks, Tychy! (lherron@imageek.york.cuny.edu) */
-		if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
-			say("DCC %s (%s %ld) request received from %s!%s [%s:%d]",
-			    type, description, Client->filesize, user, FromUserHost,
-			    inet_ntoa(Client->remote), ntohs(Client->remport));
-		else
-		        say("DCC %s (%s) request received from %s!%s [%s:%d]", 
-                             type, description, user, FromUserHost,
-                             inet_ntoa(Client->remote), ntohs(Client->remport));
-								}
-
-	    get_time(&Client->lasttime);
-	    get_time(&Client->starttime);
-	    return;
+	    /* Thanks, Tychy! (lherron@imageek.york.cuny.edu) */
+	    if ((Client->flags & DCC_TYPES) == DCC_FILEREAD)
+		say("DCC %s (%s %ld) request received from %s!%s [%s:%d]",
+		    type, description, Client->filesize, user, FromUserHost,
+		    inet_ntoa(Client->remote), ntohs(Client->remport));
+	    else
+	        say("DCC %s (%s) request received from %s!%s [%s:%d]", 
+		    type, description, user, FromUserHost,
+		    inet_ntoa(Client->remote), ntohs(Client->remport));
 	}
+
+	get_time(&Client->lasttime);
+	get_time(&Client->starttime);
+	return;
 }
 
 
@@ -2038,6 +2072,7 @@ void	dcc_check (fd_set *Readables)
 				dcc_timeout >= 0 &&
 				(now - Client->lasttime.tv_sec > dcc_timeout))
 		{
+			Client->locked++;
 			if (do_hook(DCC_LOST_LIST,"%s %s %s IDLE TIME EXCEEDED",
 				Client->user, 
 				dcc_types[Client->flags & DCC_TYPES],
@@ -2053,6 +2088,7 @@ void	dcc_check (fd_set *Readables)
 			 * good reason not to let them.
 			 */
 			Client->flags |= DCC_DELETE;
+			Client->locked--;
 		}
 
 
@@ -2098,12 +2134,14 @@ static	void	process_incoming_chat (DCC_list *Client)
 
 		Client->flags &= ~DCC_MY_OFFER;
 		Client->flags |= DCC_ACTIVE;
+		Client->locked++;
                 if (do_hook(DCC_CONNECT_LIST, "%s CHAT %s %d", Client->user,
                          inet_ntoa(remaddr.sin_addr), ntohs(remaddr.sin_port)))
 		say("DCC chat connection to %s[%s:%d] established", 
 			Client->user, inet_ntoa(remaddr.sin_addr), 
 			ntohs(remaddr.sin_port));
 		get_time(&Client->starttime);
+		Client->locked--;
 		return;
 	}
 
@@ -2119,10 +2157,12 @@ static	void	process_incoming_chat (DCC_list *Client)
 				strerror(dgets_errno));
 
 		Client->flags |= DCC_DELETE;
+		Client->locked++;
                 if (do_hook(DCC_LOST_LIST, "%s CHAT %s", 
 				Client->user, real_tmp))
         		say("DCC CHAT connection to %s lost [%s]", 
 				Client->user, real_tmp);
+		Client->locked--;
 		return;
 	}
 	case 0:		/* We do buffering, so just ignore incompletes */
@@ -2195,6 +2235,7 @@ static	void	process_incoming_chat (DCC_list *Client)
 		if (!*tmp)
 			break;
 
+		Client->locked++;
 		if (do_hook(DCC_CHAT_LIST, "%s %s", Client->user, tmp))
 		{
 			if (get_server_away(-2))
@@ -2206,6 +2247,7 @@ static	void	process_incoming_chat (DCC_list *Client)
 			}
 			put_it("=%s= %s", Client->user, tmp);
 		}
+		Client->locked--;
 		message_from(NULL, LOG_CURRENT);
 		return;
 	}
@@ -2251,16 +2293,18 @@ struct	hostent		*hp;
 	get_time(&NewClient->starttime);
 	new_open(NewClient->read);
 
+	Client->locked++;
 	if (do_hook(DCC_RAW_LIST, "%s %s N %d", NewClient->user,
 						NewClient->description,
 						Client->local_port))
-        if (do_hook(DCC_CONNECT_LIST,"%s RAW %s %d", NewClient->user,
+            if (do_hook(DCC_CONNECT_LIST,"%s RAW %s %d", NewClient->user,
                                                      NewClient->description,
                                                      Client->local_port))
 		say ("DCC RAW connection to %s on %s via %d established",
 					NewClient->description,
 					NewClient->user,
 					Client->local_port);
+	Client->locked--;
 }
 
 
@@ -2281,12 +2325,14 @@ static	void		process_incoming_raw (DCC_list *Client)
 	    case -1:
 	    {
 		Client->flags |= DCC_DELETE;
+		Client->locked++;
 		if (do_hook(DCC_RAW_LIST, "%s %s C",
 				Client->user, Client->description))
        		if (do_hook(DCC_LOST_LIST,"%s RAW %s", 
 				Client->user, Client->description))
 			say("DCC RAW connection to %s on %s lost",
 				Client->user, Client->description);
+		Client->locked--;
 		break;
 	    }
 	    case 0:
@@ -2296,10 +2342,12 @@ static	void		process_incoming_raw (DCC_list *Client)
 		if (bytesread > 0 && tmp[strlen(tmp) - 1] == '\n')
 			tmp[strlen(tmp) - 1] = '\0';
 		Client->bytes_read += bytesread;
+		Client->locked++;
 		if (do_hook(DCC_RAW_LIST, "%s %s D %s",
 				Client->user, Client->description, tmp))
 			say("Raw data on %s from %s: %s",
 				Client->user, Client->description, tmp);
+		Client->locked--;
 	    }
 	}
 	return;
@@ -2359,6 +2407,7 @@ static void		process_outgoing_file (DCC_list *Client)
 		/*
 		 * Tell the user
 		 */
+		Client->locked++;
                 if (do_hook(DCC_CONNECT_LIST, "%s SEND %s %d %s %ld",
                         Client->user, inet_ntoa(remaddr.sin_addr),
                         ntohs(remaddr.sin_port), Client->description,
@@ -2366,6 +2415,7 @@ static void		process_outgoing_file (DCC_list *Client)
 		    say("DCC SEND connection to %s[%s:%d] established", 
 			Client->user, inet_ntoa(remaddr.sin_addr), 
 			ntohs(remaddr.sin_port));
+		Client->locked--;
 
 		/*
 		 * Open up the file to be sent
@@ -2409,10 +2459,12 @@ static void		process_outgoing_file (DCC_list *Client)
 		{
 			Client->flags |= DCC_DELETE;
 
+			Client->locked++;
                         if (do_hook(DCC_LOST_LIST,"%s SEND %s CONNECTION LOST",
                                 Client->user, Client->description))
 	       		    say("DCC SEND:%s connection to %s lost",
 				Client->description, Client->user);
+			Client->locked--;
 			return;
 		}
 
@@ -2679,10 +2731,12 @@ void 	dcc_reject (char *from, char *type, char *args)
 		Client->flags |= DCC_REJECTED;
 		Client->flags |= DCC_DELETE; 
 
+		Client->locked++;
                 if (do_hook(DCC_LOST_LIST,"%s %s %s REJECTED", from, type,
                         description ? description : "<any>"))
 		    say("DCC %s:%s rejected by %s: closing", type,
 			description ? description : "<any>", from);
+		Client->locked--;
 	}
 }
 
@@ -2712,11 +2766,13 @@ static void DCC_close_filesend (DCC_list *Client, char *info)
 
 	Client->flags |= DCC_DELETE;
 
+	Client->locked++;
 	if (do_hook(DCC_LOST_LIST,"%s %s %s %s TRANSFER COMPLETE",
 		Client->user, info, Client->description, lame_ultrix))
 	     say("DCC %s:%s [%skb] with %s completed in %s sec (%s kb/sec)",
 		info, Client->description, lame_ultrix2, Client->user, 
 		lame_ultrix3, lame_ultrix);
+	Client->locked--;
 }
 
 
