@@ -854,7 +854,8 @@ __inline static	TOKEN	push_token (expr_info *c, TOKEN t)
 {
 	if (c->sp == STACKSZ - 1)
 	{
-		error("Expressions may not have more than 99 operands");
+		error("Expressions may not have more than %d operands",
+			STACKSZ);
 		return -1;
 	}
 	else
@@ -2320,6 +2321,14 @@ static  char *	after_expando_special (expr_info *c)
 		start = rest + 1;
 	}
 
+	if (c->ptr == rest)
+	{
+		yell("Erf.  I'm trying to find an lval at [%s] and I'm not "
+			"having much luck finding one.  Punting the rest of "
+			"this expression", c->ptr);
+		return NULL;
+	}
+
 	/*
 	 * All done!
 	 */
