@@ -1,11 +1,11 @@
 #include "config.h"
-#if defined(NEED_GLOB) && defined(INCLUDE_GLOB_FUNCTION)
+#if defined(NEED_GLOB)
 
 /*
  * Copyright (c) 1989, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
- * This code is derived from software contributed to Berkeley by
+ * This code is derived from software contributed to Berkeley by 
  * Guido van Rossum.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -16,15 +16,11 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND 
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
@@ -61,13 +57,14 @@
  *	expand ~user/foo to the /home/dir/of/user/foo
  * GLOB_BRACE:
  *	expand {1,2}{a,b} to 1a 1b 2a 2b 
+ * GLOB_INSENSITIVE:
+ *	Globbing occurs without regard to case of the letters.
  * gl_matchc:
  *	Number of matches in the current invocation of glob.
  */
 
 #include <sys/param.h>
 #include <sys/stat.h>
-
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
@@ -77,7 +74,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "ircaux.h"
+#include "irc.h"
 #ifndef MAXPATHLEN
 #define MAXPATHLEN PATHSIZE
 #endif
@@ -103,6 +100,7 @@
 
 #define	M_QUOTE		0x8000
 #define	M_PROTECT	0x4000
+#define M_ANYCASE	0x2000		/* EPIC ADD */
 #define	M_MASK		0xffff
 #define	M_ASCII		0x00ff
 
@@ -133,7 +131,7 @@ static int	 	globextend (const Char *, glob_t *);
 static const Char *	globtilde (const Char *, Char *, glob_t *);
 static int	 	globexp1 (const Char *, glob_t *);
 static int	 	globexp2 (const Char *, const Char *, glob_t *, int *);
-static int	 	match (Char *, Char *, Char *);
+static int	 	globmatch (Char *, Char *, Char *, int);
 
 int bsd_glob		(	const char *pattern,
 				int flags, 
@@ -145,7 +143,8 @@ int bsd_glob		(	const char *pattern,
 	Char *bufnext, *bufend, patbuf[MAXPATHLEN+1];
 
 	patnext = (u_char *) pattern;
-	if (!(flags & GLOB_APPEND)) {
+	if (!(flags & GLOB_APPEND)) 
+	{
 		pglob->gl_pathc = 0;
 		pglob->gl_pathv = NULL;
 		if (!(flags & GLOB_DOOFFS))
@@ -157,11 +156,15 @@ int bsd_glob		(	const char *pattern,
 
 	bufnext = patbuf;
 	bufend = bufnext + MAXPATHLEN;
-	if (flags & GLOB_QUOTE) {
+	if (flags & GLOB_QUOTE) 
+	{
 		/* Protect the quoted characters. */
 		while (bufnext < bufend && (c = *patnext++) != EOS) 
-			if (c == QUOTE) {
-				if ((c = *patnext++) == EOS) {
+		{
+			if (c == QUOTE) 
+			{
+				if ((c = *patnext++) == EOS) 
+				{
 					c = QUOTE;
 					--patnext;
 				}
@@ -169,10 +172,12 @@ int bsd_glob		(	const char *pattern,
 			}
 			else
 				*bufnext++ = c;
+		}
 	}
-	else 
-	    while (bufnext < bufend && (c = *patnext++) != EOS) 
-		    *bufnext++ = c;
+	else
+		while (bufnext < bufend && (c = *patnext++) != EOS) 
+			*bufnext++ = c;
+
 	*bufnext = EOS;
 
 	if (flags & GLOB_BRACE)
@@ -226,6 +231,7 @@ static int globexp2	(	const Char *ptr,
 
 	/* Find the balanced brace */
 	for (i = 0, pe = ++ptr; *pe; pe++)
+	{
 		if (*pe == LBRACKET) 
 		{
 			/* Ignore everything between [] */
@@ -248,6 +254,7 @@ static int globexp2	(	const Char *ptr,
 				break;
 			i--;
 		}
+	}
 
 	/* Non matching braces; just glob the pattern */
 	if (i != 0 || *pe == EOS) 
@@ -257,6 +264,7 @@ static int globexp2	(	const Char *ptr,
 	}
 
 	for (i = 0, pl = pm = ptr; pm <= pe; pm++)
+	{
 		switch (*pm) 
 		{
 		case LBRACKET:
@@ -310,6 +318,7 @@ static int globexp2	(	const Char *ptr,
 		default:
 			break;
 		}
+	}
 	*rv = 0;
 	return 0;
 }
@@ -338,7 +347,8 @@ static const Char *globtilde	(	const Char *pattern,
 
 	*h = EOS;
 
-	if (((char *) patbuf)[0] == EOS) {
+	if (((char *) patbuf)[0] == EOS) 
+	{
 		/* 
 		 * handle a plain ~ or ~/ by expanding $HOME 
 		 * first and then trying the password file
@@ -354,7 +364,7 @@ static const Char *globtilde	(	const Char *pattern,
 		/*
 		 * Expand a ~user
 		 */
-		if ((pwd = getpwnam((char*) patbuf)) == NULL)
+		if ((pwd = getpwnam((char *)patbuf)) == NULL)
 			return pattern;
 		else
 			h = pwd->pw_dir;
@@ -382,9 +392,12 @@ static const Char *globtilde	(	const Char *pattern,
 static int glob0		(	const Char *pattern,
 					glob_t *pglob		)
 {
-	const Char *qpatnext;
-	int c, err, oldpathc;
-	Char *bufnext, patbuf[MAXPATHLEN+1];
+	const Char *	qpatnext;
+	int 		c;
+	int		err;
+	int		oldpathc;
+	Char *		bufnext;
+	Char		patbuf[MAXPATHLEN+1];
 
 	qpatnext = globtilde(pattern, patbuf, pglob);
 	oldpathc = pglob->gl_pathc;
@@ -399,7 +412,8 @@ static int glob0		(	const Char *pattern,
 			c = *qpatnext;
 			if (c == NOT)
 				++qpatnext;
-			if (*qpatnext == EOS || g_strchr((Char *) qpatnext+1, RBRACKET) == NULL) 
+			if (*qpatnext == EOS || 
+			    g_strchr((Char *) qpatnext+1, RBRACKET) == NULL) 
 			{
 				*bufnext++ = LBRACKET;
 				if (c == NOT)
@@ -413,7 +427,8 @@ static int glob0		(	const Char *pattern,
 			do 
 			{
 				*bufnext++ = CHAR(c);
-				if (*qpatnext == RANGE && (c = qpatnext[1]) != RBRACKET) 
+				if (*qpatnext == RANGE && 
+				    (c = qpatnext[1]) != RBRACKET) 
 				{
 					*bufnext++ = M_RNG;
 					*bufnext++ = CHAR(c);
@@ -431,7 +446,8 @@ static int glob0		(	const Char *pattern,
 			break;
 		case STAR:
 			pglob->gl_flags |= GLOB_MAGCHAR;
-			/* collapse adjacent stars to one, 
+			/* 
+			 * collapse adjacent stars to one, 
 			 * to avoid exponential behavior
 			 */
 			if (bufnext == patbuf || bufnext[-1] != M_ALL)
@@ -459,15 +475,18 @@ static int glob0		(	const Char *pattern,
 	       !(pglob->gl_flags & GLOB_MAGCHAR))))
 		return(globextend(pattern, pglob));
 	else if (!(pglob->gl_flags & GLOB_NOSORT)) 
+	{
+	    if (pglob->gl_pathv)
 		qsort(pglob->gl_pathv + pglob->gl_offs + oldpathc,
 		    pglob->gl_pathc - oldpathc, sizeof(char *), compare);
+	}
 	return(0);
 }
 
 static int compare		(	const void *p,
 					const void *q		)
 {
-	return(strcmp(*(char **)p, *(char **)q));
+	return (strcmp(*(char **)p, *(char **)q));
 }
 
 static int glob1		(	Char *pattern,
@@ -587,10 +606,12 @@ static int glob3		(	Char *pathbuf,
 		readdirfunc = pglob->gl_readdir;
 	else
 		readdirfunc = readdir;
+
 	while ((dp = (*readdirfunc)(dirp))) 
 	{
 		register u_char *sc;
 		register Char *dc;
+		int	nocase = 0;
 
 		/* Initial DOT must be matched literally. */
 		if (dp->d_name[0] == DOT && *pattern != DOT)
@@ -598,7 +619,12 @@ static int glob3		(	Char *pathbuf,
 		for (sc = (u_char *) dp->d_name, dc = pathend; 
 		     (*dc++ = *sc++) != EOS;)
 			continue;
-		if (!match(pathend, pattern, restpattern)) 
+
+
+		if (pglob->gl_flags & GLOB_INSENSITIVE)
+			nocase = 1;
+
+		if (!globmatch(pathend, pattern, restpattern, nocase)) 
 		{
 			*pathend = EOS;
 			continue;
@@ -635,7 +661,7 @@ static int globextend		(	const Char *path,
 {
 	register char **pathv;
 	register int i;
-	u_int newsize;
+	unsigned newsize;
 	char *copy;
 	const Char *p;
 
@@ -671,9 +697,9 @@ static int globextend		(	const Char *path,
  * pattern matching function for filenames.  Each occurrence of the *
  * pattern causes a recursion level.
  */
-static int match		(	register Char *name,
+static int globmatch		(	register Char *name,
 					register Char *pat,
-					register Char *patend		)
+					register Char *patend, int nocase )
 {
 	int ok, negate_range;
 	Char c, k;
@@ -687,10 +713,11 @@ static int match		(	register Char *name,
 			if (pat == patend)
 				return(1);
 			do 
-			    if (match(name, pat, patend))
+			    if (globmatch(name, pat, patend, nocase))
 				    return(1);
 			while (*name++ != EOS);
 			return(0);
+
 		case M_ONE:
 			if (*name++ == EOS)
 				return(0);
@@ -715,12 +742,28 @@ static int match		(	register Char *name,
 				return(0);
 			break;
 		default:
-			if (*name++ != c)
-				return(0);
+		{
+			if (nocase)
+			{
+				if (toupper((int)CHAR(*name)) != 
+				    toupper((int)CHAR(c)))
+					return 0;
+			}
+			else
+			{
+				if (*name != c)
+					return 0;
+			}
+			name++;
 			break;
 		}
+		}
 	}
-	return(*name == EOS);
+
+	if (*name == EOS)
+		return 1;
+
+	return 0;
 }
 
 /* Free allocated data belonging to a glob_t structure. */
