@@ -13,14 +13,14 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC4-0.9.9";
-const char useful_info[] = "epic4 0 9 9";
+const char irc_version[] = "EPIC4-0.9.13";
+const char useful_info[] = "epic4 0 9 13";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20000929";
+const char internal_version[] = "20001130";
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -28,7 +28,7 @@ const char internal_version[] = "20000929";
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Diligence";
+const char ridiculous_version_name[] = "Mollify";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -910,6 +910,7 @@ static	struct	timeval cursor_timeout,
 	if (need_redraw)
 		refresh_screen('\0', NULL);
 
+	window_check_channels();
 	alloca(0);
 	caller[level] = NULL;
 	level--;
