@@ -105,6 +105,13 @@ void set_log_file (char *filename)
 			expand = expand_twiddle(filename);
 		else
 			expand = expand_twiddle(get_string_var(LOGFILE_VAR));
+
+		if (!expand)
+		{
+			say("SET LOGFILE: No such user");
+			return;
+		}
+
 		set_string_var(LOGFILE_VAR, expand);
 		new_free(&expand);
 		if (irclog_fp)
