@@ -139,7 +139,8 @@ int	traverse_all_channels (Channel **ptr, int server)
 	 * important.  So we have to go find a window that thinks its on 
 	 * the same server we are.
 	 */
-	if (!(*ptr)->window || (*ptr)->window->server != (*ptr)->server)
+	if (!(*ptr)->window || (is_server_connected((*ptr)->window->server) &&
+				(*ptr)->window->server != (*ptr)->server))
 	{
 		Window *w = NULL;
 		(*ptr)->window = NULL;
@@ -167,9 +168,6 @@ int	traverse_all_channels (Channel **ptr, int server)
 		 * list, but that came up when the server was already
 		 * connected, which wasnt terribly useful.
 		 */
-#if 0
-		if (x_debug & DEBUG_CHANNELS)
-#endif
 		if (!(*ptr)->window)
 		{
 			/*
@@ -261,7 +259,7 @@ static void 	clear_channel (Channel *chan)
 	list->max = list->max_alloc = 0;
 }
 
-/* Channel destructor */
+/* Channel destructor -- caller must free "chan". */
 static void 	destroy_channel (Channel *chan)
 {
 	if (chan != channel_list)
@@ -282,13 +280,29 @@ static void 	destroy_channel (Channel *chan)
 
 	if (chan->window)
 	{
+		/*
+		 * If 'chan' is a current channel (and believe me, there is
+		 * no easy way to tell short of looking), then we will want
+		 * to give the window a chance to do something about it.  So
+		 * we call channel_going_away to tell the window that this 
+		 * channel's days are numbered.  If this is a current channel
+		 * then it will reset it's current channel to some other
+		 * channel it has, or to nothing, as the case may be.  It is
+		 * customary for the window to arrange for this channel's
+		 * window to be set to NULL if this was indeed the current
+		 * channel, so we *still* have to test to make sure that this
+		 * channel is not the current channel when this returns (ugh).
+		 */
 		channel_going_away(chan->window, chan->channel);
 
 		/*
-	 	 * If this was the last channel on that window, be more 
-		 * forceful about removing this channel.
-	 	 */
-		if (chan->window->current_channel)
+		 * So if 'chan->window' is not null, then this was probably
+		 * not a current channel.  But if 'current_channel' exists,
+		 * and it's pointing at us, look out!  If this ever does
+		 * happen, we take matters into our own hands and tell the
+		 * window to get lost.
+		 */
+		if (chan->window && chan->window->current_channel)
 		  if (!my_stricmp(chan->window->current_channel, chan->channel))
 			set_channel_by_refnum(chan->window->refnum, NULL);
 	}
@@ -320,6 +334,7 @@ void 	add_channel (const char *name, int server)
 	Window	*was_window = NULL;
 	Window	*tmp = NULL;
 
+	/* Bogus dependancy on 'from_server == server' here. */
 	was_current = is_current_channel(name, 0);
 
 	if ((new_c = find_channel(name, server)))
@@ -350,6 +365,7 @@ void 	add_channel (const char *name, int server)
 	else
 		new_c = create_channel(name, server);
 
+	new_c->inactive = 0;		/* Channel no longer is a placeholder */
 	new_c->waiting = 1;		/* This channel is "syncing" */
 	get_time(&new_c->join_time);
 
@@ -1278,25 +1294,29 @@ void 	change_server_channels (int old_s, int new_s)
 void 	destroy_server_channels (int server)
 {
 	Channel	*tmp = NULL;
+	int	reset = 0;
 
-	if (server < 0) return;		/* Sanity check */
+	if (server < 0)
+		return;		/* Sanity check */
 
-	/* 
-	 * 'tmp' gets reset to NULL after every destructive act,
-	 * thus restarting the traversal.  At some point, we walk through
-	 * the list without deleting anything then we stop.
+	/*
+	 * Regretably, the referntial integrity of the channels is broken
+	 * at this point -- meaning that we cannot use traverse_all_channels
+	 * which attempts to restore referntial integrity -- but we do not
+	 * want that!  So we have to slog through this on our own and kill
+	 * off the channels without the assistance of traverse_all_channels.
 	 */
-	while (traverse_all_channels(&tmp, server))
+	for (tmp = channel_list; tmp; 
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
 	{
+		reset = 0;
+		if (tmp->server != server)
+			continue;
 		destroy_channel(tmp);
 		new_free((char **)&tmp);
+		reset = 1;
 	}
-
-	/* This looks very pointless -- and dangerous */
-	/* XXX Try to figure out if this is _really_ neccesary to do here. */
-	/* Try to figure out what channel to use... >;-) */
-	if (!dead && current_window->server != server)
-		switch_channels(0, 0);
+	window_check_channels();
 }
 
 
@@ -1319,6 +1339,11 @@ void 	reconnect_all_channels (void)
 
 	while (traverse_all_channels(&tmp, from_server))
 	{
+		if (!tmp->inactive)
+			yell("Ack.  Reconnecting channel [%s] on server [%d] "
+			     "but it isn't inactive!", 
+				tmp->channel, tmp->server);
+
 		/* Ugh.  Hybrid makes me do this. */
 		m_s3cat(&channels, ",", tmp->channel);
 		m_s3cat(&keys, ",", tmp->key ? tmp->key : empty_string);
@@ -1451,6 +1476,7 @@ void 	set_channel_window (Window *window, const char *channel)
 void	unset_window_current_channel (Window *old_w, Window *new_w)
 {
 	Channel *tmp = NULL;
+	char	*chan;
 
 	if (old_w->server < 0) return;		/* Sanity check */
 
@@ -1458,8 +1484,9 @@ void	unset_window_current_channel (Window *old_w, Window *new_w)
 	{
 		if (tmp->window == old_w && !my_stricmp(tmp->channel, old_w->current_channel))
 		{
-			new_free(&old_w->current_channel);
-			tmp->window = new_w;
+			chan = LOCAL_COPY(old_w->current_channel);
+			set_channel_by_refnum(old_w->refnum, NULL);
+			set_channel_by_refnum(new_w->refnum, chan);
 			return;
 		}
 	}
@@ -1479,7 +1506,7 @@ void	reset_window_current_channel (Window *w)
 	{
 		if (tmp->window == w)
 		{
-			malloc_strcpy(&w->current_channel, tmp->channel);
+			set_channel_by_refnum(w->refnum, tmp->channel);
 			return;
 		}
 	}
@@ -1493,14 +1520,31 @@ void	reassign_window_channels (Window *window)
 {
 	Channel *tmp = NULL;
 	Window *w = NULL;
+	int	caution = 0;
 
-	if (window->server < 0) return;		/* Sanity check */
+	if (window->server < 0 && window->last_server < 0)
+		caution = 1;
 
-	while (traverse_all_channels(&tmp, window->server))
+	for (tmp = channel_list; tmp; tmp = tmp->next)
 	{
 		if (tmp->window != window)
 			continue;
 
+		if (caution)
+		{
+			yell("Deleting channels for disconnected window [%d] "
+				"-- caution!", window->refnum);
+			caution = 0;
+		}
+
+		if (tmp->server != window->server && 
+			tmp->server != window->last_server)
+		{
+			yell("Channel [%s:%d] is connected to window [%d] "
+				"which is apparantly on another server", 
+				tmp->channel, tmp->server, window->refnum);
+		}
+
 		/*
 		 * If all else fails, and we can't find another window
 		 * for this channel, its window will be NULL and that
@@ -1648,3 +1692,126 @@ static int	match_chan_with_id (const char *chan, const char *match)
 	return my_stricmp(chan, match);
 }
 
+
+/*
+ *
+ */
+void	channel_check_windows (void)
+{
+	Channel	*tmp = NULL;
+	Window *w = NULL;
+	int	reset = 0;
+
+	/* Do test #3 -- check for windowless channels */
+	for (tmp = channel_list; tmp; 
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
+	{
+		reset = 0;
+
+		if (tmp->window)
+			continue;
+
+		w = NULL;
+		while (traverse_all_windows(&w))
+		{
+		    if (w->server == tmp->server)
+		    {
+			yell("Repaired referential integrity failure: "
+			     "channel [%s] on server [%d] was not "
+			     "connected to any window; moved to "
+			     "window refnum [%d]",
+				tmp->channel, tmp->server, 
+				w->refnum);
+			tmp->window = w;
+			reset = 1;
+			break;
+		    }
+		}
+
+		if (tmp->window == NULL)
+		{
+			yell("Repaired referential integrity failure: "
+			     "server [%d] has channels, but no "
+			     "windows -- throwing away all of these "
+			     "server's channels", tmp->server);
+			destroy_server_channels(tmp->server);
+			reset = 1;
+		}
+	}
+
+	/* Do test #4 -- check for windows from other servers */
+	for (tmp = channel_list; tmp; 
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
+	{
+		if (tmp->window == NULL)
+			panic("I thought we just checked for this! [1]");
+
+		if (tmp->window->server == -1 && 
+				tmp->server == tmp->window->last_server)
+			continue;			/* This is OK. */
+
+		if (tmp->server != tmp->window->server)
+			panic("Referential integrity failure: "
+			      "Channel [%s] on server [%d] is connected "
+			      "to window [%d] on server [%d]",
+				tmp->channel, tmp->server, 
+				tmp->window->refnum, tmp->window->server);
+	}
+
+	/* Do test #5 -- check for bogus windows */
+	for (tmp = channel_list; tmp; 
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
+	{
+		if (tmp->window == NULL)
+			panic("I thought we just checked for this! [2]");
+
+		if (tmp->window != get_window_by_refnum(tmp->window->refnum))
+			panic("Referential integrity failure: "
+			      "Channel [%s] on server [%d] is connected "
+			      "to window [%d] on server [%d] "
+			      "that doesn't exist any more!",
+				tmp->channel, tmp->server, 
+				tmp->window->refnum, tmp->window->server);
+	}
+
+	/* Do test #6 -- check for current channel-less windows */
+	for (tmp = channel_list; tmp;
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
+	{
+		if (tmp->window == NULL)
+			panic("I thought we just checked for this! [3]");
+
+		/* Defer this check until after we have attempted rejoin */
+		if (did_server_rejoin_channels(tmp->server))
+		{
+		  if (tmp->window->current_channel == NULL &&
+			tmp->window->waiting_channel == NULL)
+		  {
+		       yell("Repaired referential integrity failure: "
+			"Window [%d] has channels, but no current or waiting "
+			"channel -- making [%s] the current channel.",
+				tmp->window->refnum, tmp->channel);
+		       set_channel_by_refnum(tmp->window->refnum, tmp->channel);
+		  }
+		}
+	}
+
+	/* Do test #7 -- check for abandoned channels */
+	for (tmp = channel_list; tmp;
+		reset ? (tmp = channel_list) : (tmp = tmp->next))
+	{
+		if (tmp->window == NULL)
+			panic("I thought we just checked for this! [4]");
+
+		if (did_server_rejoin_channels(tmp->server) && tmp->inactive)
+			panic("Channel [%s] on server [%d] is inactive "
+				"even though this server is connected!",
+				tmp->channel, tmp->server);
+		if (!did_server_rejoin_channels(tmp->server) && !tmp->inactive)
+			panic("Channel [%s] on server [%d] is NOT inactive "
+				"even though this server is NOT connected!",
+				tmp->channel, tmp->server);
+	}
+
+	return;
+}
