@@ -370,6 +370,7 @@ void 	got_initial_version_28 (char **ArgList)
 	}
 
 	reconnect_all_channels();
+	server_did_rejoin_channels(from_server);
 	message_from(NULL, LOG_CRAP);
 	reinstate_user_modes();
 
@@ -405,6 +406,7 @@ void 	got_initial_version_28 (char **ArgList)
 	update_all_status();
 	do_hook(CONNECT_LIST, "%s %d %s", get_server_name(from_server),
 		get_server_port(from_server), get_server_itsname(from_server));
+	window_check_channels();
 }
 
 int 	kill_message (const char *from, char *cline)
