@@ -118,6 +118,34 @@ void 	display_msg (char *from, char **ArgList)
  * numbered_command: does (hopefully) the right thing with the numbered
  * responses from the server.  I wasn't real careful to be sure I got them
  * all, but the default case should handle any I missed (sorry) 
+ *
+ * The format of a numeric looks like so:
+ *
+ *	:server-name XXX our-nick Arg1 Arg2 Arg3 ... :ArgN
+ *
+ * The last argument traditionally has a colon before it, but this is not
+ * compulsary.  The BreakArgs function has already broken this up into 
+ * words for us, so that what we get, looks like this:
+ *
+ *	server-name	-> 	from parameter
+ *	XXX		->	comm parameter
+ *	our-nick	->	ArgList[0]
+ *	Arg1		->	ArgList[1]
+ *	Arg2		->	ArgList[2]
+ *	...			...
+ *
+ * BUT!  There's a wrinkle in the ointment.  The first thing we do is slurp
+ * up ArgList[0] (our-nick) and put it in 'user'.  Then we increment the 
+ * ArgList array, so what we actually end up with is:
+ *
+ *	server-name	-> 	from parameter
+ *	XXX		->	comm parameter
+ *	our-nick	->	user
+ *	Arg1		->	ArgList[0]
+ *	Arg2		->	ArgList[1]
+ *	...			...
+ *	ArgN		->	ArgList[N-1]
+ *	NULL		->	ArgList[N]
  */
 void 	numbered_command (char *from, int comm, char **ArgList)
 {
@@ -476,11 +504,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 			display_msg(from, ArgList);
 		break;
 
+        /* ":%s 401 %s %s :No such nick/channel" */
 	case 401:		/* #define ERR_NOSUCHNICK       401 */
 	{
 		PasteArgs(ArgList, 1);
-		if (do_hook(current_numeric, "%s %s %s", from, ArgList[0], ArgList[1]))
-			put_it("%s %s: %s", numeric_banner(), ArgList[0], ArgList[1]);
+		if (do_hook(current_numeric, "%s %s %s", from, 
+						ArgList[0], ArgList[1]))
+			put_it("%s %s: %s", numeric_banner(), 
+						ArgList[0], ArgList[1]);
 		notify_mark(ArgList[0], 0, 0);
 
 		if (get_int_var(AUTO_WHOWAS_VAR))
@@ -494,6 +525,89 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		}
 		break;
 	}
+
+	/* Broken dalnet extended who replies. */
+        /* ":%s 402 %s %s :No such server" */
+	case 402:
+	{
+		PasteArgs(ArgList, 1);
+		if (do_hook(current_numeric, "%s %s %s", from, 
+					ArgList[0], ArgList[1]))
+			display_msg(from, ArgList);
+
+		/* 
+		 * Some broken servers send this instead of a 315 numeric
+		 * when a who request has been completed.
+		 */
+		fake_who_end(from, ArgList[0]);
+		break;
+	}
+
+	/* Yet more broken dalnet extended who replies. */
+	/* ":%s 522 %s :/WHO Syntax incorrect, use /who ? for help" */
+        /* ":%s 523 %s :Error, /who limit of %d exceed." */
+	case 522:
+	case 523:
+	{
+		PasteArgs(ArgList, 0);
+		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+			display_msg(from, ArgList);
+
+		/* 
+		 * This dalnet error message doesn't even give us the
+		 * courtesy of telling us which who request was in error,
+		 * so we have to guess.  Whee.
+		 */
+		fake_who_end(from, NULL);
+		break;
+	}
+
+	case 403:		/* #define ERR_NOSUCHCHANNEL    403 */
+	{
+		const char	*s;
+
+		PasteArgs(ArgList, 1);
+
+		/* Do not accept 403's from remote servers. */
+		s = get_server_name(from_server);
+		if (my_strnicmp(s, from, strlen(s)))
+			break;
+
+		/* Some broken servers may send back an empty reply. */
+		if (!ArgList[0])
+		{
+			if (do_hook(current_numeric, "%s *", from))
+				put_it("%s You did not specify a channel", 
+					numeric_banner());
+			break;
+		}
+
+		if (do_hook(current_numeric, "%s %s %s", from, 
+					ArgList[0], ArgList[1]))
+			display_msg(from, ArgList);
+
+		/* 
+		 * Some broken servers send this instead of a 315 numeric
+		 * when a who request has been completed.
+		 */
+		if (fake_who_end(from, ArgList[0]))
+			break;
+
+		/*
+		 * If you try to JOIN or PART the "*" named channel, as may
+		 * happen if epic gets confused, the server may tell us that
+		 * channel does not exist.  But it would be death to try to 
+		 * destroy that channel as epic will surely do the wrong thing!
+		 * Otherwise, we somehow tried to reference a channel that
+		 * this server claims does not exist; we blow the channel away
+		 * for good measure.
+		 */
+		if (strcmp(ArgList[0], "*"))
+			remove_channel(ArgList[0], from_server);
+
+		break;
+	}
+
 	case 421:		/* #define ERR_UNKNOWNCOMMAND   421 */
 	{
 		if (check_server_redirect(ArgList[0]))
@@ -533,11 +647,16 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * Well, there are at least four different, occasionally
 		 * scrutable ways we can get this numeric.
 		 *
-		 * 1) On ircnet -- As an unregistered user, the NICK that
+		 * 1a) On ircnet -- As an unregistered user, the NICK that
 		 *	we are trying to register was used in the past 90
 		 *	seconds or so.  The server expects us to send
 		 *	another NICK request.
 		 *		ARGV[0] IS NICK, REGISTERED IS NO
+		 * 1b) On ircnet -- As a registered user, the NICK that 
+		 *	we are trying to register was used in the past 90
+		 *	seconds or so.  The server expects us not to do
+		 * 	anything (like a 432 numeric).
+		 *		ARGV[0] IS NICK, REGISTERED IS YES
 		 * 2) On ircnet -- As a registered user, we are trying to
 		 *	join a channel that was netsplit in the past 24 hours
 		 *	or so.  The server expects us not to do anything.
@@ -552,6 +671,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 *	too short a time.  The server expects us not to do
 		 *	anything.
 		 *		ARGV[0] IS ERROR, ARGV[1] IS NULL.
+		 *	I understand this numeric will be moving to 439.
 		 */
 
 		/*
@@ -561,6 +681,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		{
 			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 				display_msg(from, ArgList);
+			accept_server_nickname(from_server, user);
 			break;
 		}
 
@@ -572,6 +693,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 */
 		if (is_channel(ArgList[0]))
 		{
+		    /* XXX Is this really neccesary? */
 		    if (!im_on_channel(ArgList[0], from_server))
 			remove_channel(ArgList[0], from_server);
 
@@ -585,8 +707,10 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * If we are registered, abort the nick change and
 		 * hope for the best.
 		 */
-		else if (is_server_connected(from_server))
+		if (is_server_connected(from_server))
 		{
+			if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+				display_msg(from, ArgList);
 			accept_server_nickname(from_server, user);
 			break;
 		}
@@ -597,6 +721,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 */
 		/* FALLTHROUGH */
 	}
+
 	case 433:		/* #define ERR_NICKNAMEINUSE    433 */ 
 	case 438:		/* EFnet/TS4 "nick collision" numeric 438 */
 	case 453:		/* EFnet/TS4 "nickname lost" numeric 453 */
@@ -617,6 +742,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	case 439:		/* Comstud's "Can't change nickname" */
+	{
+		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
+			display_msg(from, ArgList);
+		accept_server_nickname(from_server, user);
+		break;
+	}
+
 	case 442:		/* #define ERR_NOTONCHANNEL	442 */
 	{
 		const char *	s;
@@ -862,33 +995,6 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 			break;
 		}
 
-		case 403:		/* #define ERR_NOSUCHCHANNEL    403 */
-		{
-			const char	*s;
-
-			if (!ArgList[1])
-				break;
-
-			PasteArgs(ArgList, 1);
-			s = get_server_name(from_server);
-			if (!my_strnicmp(s, from, strlen(s)))
-			{
-				/* hack cause channel may be null */
-				if (ArgList[0])
-				{
-					/*
-					 * Dont blindly remove channel '*' (cb)
-					 */
-					if (strcmp(ArgList[0], "*"))
-						remove_channel(ArgList[0], from_server);
-					put_it("%s %s %s", numeric_banner(), ArgList[0], ArgList[1]);
-				}
-				else
-					put_it("%s You did not specify a channel", numeric_banner());
-			}
-			break;
-		}
-
 		case 471:		/* #define ERR_CHANNELISFULL    471 */
 		case 473:		/* #define ERR_INVITEONLYCHAN   473 */
 		case 474:		/* #define ERR_BANNEDFROMCHAN   474 */
