@@ -637,7 +637,7 @@ static void 	p_invite (char *from, char **ArgList)
  * NOTE: If you want to change it, you *will* have to do it yourself.
  * Im quite serious.  Im not going to change this.
  */
-static void p_kill (char *from, char **ArgList)
+static void	p_kill (char *from, char **ArgList)
 {
 	/* 
 	 * Bogorific Microsoft Exchange ``IRC'' server sends out a KILL
@@ -666,8 +666,9 @@ static void p_kill (char *from, char **ArgList)
 
 	if (strchr(from, '.'))
         {
-		say("Server [%s] has rejected you (probably due "
-			"to a nick collision)", from);
+		say("Server [%s] has rejected you. (%s)",
+			from, ArgList[1] ? ArgList[1] : 
+			"probably due to a nick collision");
 		get_connected(from_server, from_server);
 	}
 	else
@@ -720,7 +721,7 @@ static void p_silence (char *from, char **ArgList)
 	if (do_hook(SILENCE_LIST, "%c %s", mag, target))
 	{
 		if (mag == '+')
-			say ("You will no longer recieve msgs from %s", target);
+			say ("You will no longer receive msgs from %s", target);
 		else if (mag == '-')
 			say ("You will now recieve msgs from %s", target);
 		else
