@@ -383,9 +383,6 @@ void 	add_to_window (Window *window, const unsigned char *str)
 		}
 		term_flush();
 
-		if (must_free)
-			new_free(&str);
-
 		/*
 		 * This used to be in rite(), but since rite() is a general
 		 * purpose function, and this stuff really is only intended
@@ -398,6 +395,7 @@ void 	add_to_window (Window *window, const unsigned char *str)
 			/*
 			 * This is for archon -- he wanted a way to have 
 			 * a hidden window always beep, even if BEEP is off.
+			 * XXX -- str has already been freed here! ACK!
 			 */
 			if (window->beep_always && strchr(str, '\007'))
 			{
@@ -434,6 +432,9 @@ void 	add_to_window (Window *window, const unsigned char *str)
 			}
 		}
 
+		if (must_free)
+			new_free(&str);
+
 		cursor_in_display(window);
 	}
 }
@@ -2709,7 +2710,7 @@ unsigned char *strip_ansi (const unsigned char *str)
 			case ('F') : case ('H') : case ('M') :
 			case ('N') : case ('O') : case ('Z') :
 			case ('l') : case ('m') : case ('n') :
-/* { */			case ('o') : case ('|') : case ('}') :
+			case ('o') : case ('|') : case ('}') :
 			case ('~') : case ('c') :
 			{
 				break;		/* Dont do anything */
