@@ -653,7 +653,7 @@ BUILT_IN_COMMAND(servercmd)
 	 */
 	else
 	{
-		int j = from_server;
+		int	j = from_server;
 
 		clear_reconnect_counts();
 		i = find_server_refnum(server, &args);
@@ -1094,7 +1094,7 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
 
 		/*
 		 * If 'new_conn' is 0, and 'old_server' is -1, then we're
-		 * doing something like a /server in a disconnecetd window.
+		 * doing something like a /server in a disconnected window.
 		 * That's no sweat.  We will just try to figure out what
 		 * server this window was last connected to, and move all
 		 * of those windows over to this new server.
@@ -1191,7 +1191,14 @@ int 	get_connected (int new, int old)
 	}
 
 	/*
-	 * Otherwise, our prior state was unconnected.  Tell the user
+	 * In any situation, if 'old' is not connected at this point, then
+	 * we need to throw away it's channels.
+	 */
+	if (!is_server_connected(old))
+		destroy_server_channels(old);
+
+	/*
+	 * Our prior state was unconnected.  Tell the user
 	 * that we give up and tough luck.
 	 */
 	if (do_hook(DISCONNECT_LIST, "Unable to connect to a server"))
@@ -1249,6 +1256,7 @@ void	close_server (int old, const char *message)
 	server_list[old].oper = 0;
 	server_list[old].registration_pending = 0;
 	server_list[old].connected = 0;
+	server_list[old].rejoined_channels = 0;
 	new_free(&server_list[old].nickname);
 	new_free(&server_list[old].s_nickname);
 
@@ -1601,6 +1609,7 @@ void 	server_is_connected (int sic_index, int value)
 
 	server_list[sic_index].connected = value;
 	server_list[sic_index].registration_pending = 0;
+	server_list[sic_index].rejoined_channels = 0;
 	if (value)
 	{
 		server_list[sic_index].eof = 0;
@@ -1608,6 +1617,22 @@ void 	server_is_connected (int sic_index, int value)
 	}
 }
 
+void	server_did_rejoin_channels (int sic_index)
+{
+	if (sic_index < 0 || sic_index >= number_of_servers)
+		return;
+	server_list[sic_index].rejoined_channels = 1;
+}
+
+int	did_server_rejoin_channels (int sic_index)
+{
+	if (sic_index < 0 || sic_index >= number_of_servers)
+		return 0;
+	if (server_list[sic_index].connected == 0)
+		return 0;
+	return server_list[sic_index].rejoined_channels;
+}
+
 BUILT_IN_COMMAND(disconnectcmd)
 {
 	char	*server;
