@@ -592,9 +592,7 @@ void	who_end (char *from, char **ArgList)
 
 	do
 	{
-		/*
-		 * Defer to another function, if neccesary.
-		 */
+		/* Defer to another function, if neccesary.  */
 		if (new_w->end)
 			new_w->end(from, ArgList);
 		else
@@ -613,6 +611,70 @@ void	who_end (char *from, char **ArgList)
 	who_queue_pop();
 }
 
+/*
+ * Thanks be to the Dalnet coding team who, in their infinite wisdom,
+ * decided to start returning error codes, rather than empty who replies,
+ * for who requests that refered to channels or server or whatnot that
+ * do not exist.  So as to avoid corrupting the WHO queue, this function is
+ * called to give us the opportunity to clean up after any who requests that
+ * may be canceled because of the above lamage.
+ *
+ * Thanks be to the Dalnet coding team for returning error codes that do not
+ * tell you what the original request was, making it neigh unto impossible
+ * to correctly match up error codes to requests.  Gee whiz, you wouldn't
+ * think it would be that hard to get this right.
+ */
+int	fake_who_end (char *from, char *who_target)
+{
+	WhoEntry 	*new_w = who_queue_top();
+	char 		buffer[1025];
+
+	if (who_whine)
+		who_whine = 0;
+	if (!new_w)
+		return 0;	
+
+	/* Only honor these for dalnet extended requests. */
+	if (new_w->dalnet_extended == 0)
+		return 0;
+
+	if (who_target != NULL)
+	{
+		/*
+		 * So 'who_target' isn't NULL here.  Make sure it's a 
+		 * legitimate match to our current top of who request.
+		 */
+		if (strcmp(new_w->who_target, who_target))
+			return 0;
+	}
+
+	do
+	{
+		/* Defer to another function, if neccesary.  */
+		if (new_w->end)
+		{
+			char *fake_ArgList[3];
+
+			/* Fabricate a fake argument list */
+			fake_ArgList[0] = new_w->who_target;
+			fake_ArgList[1] = "fake_who_end";
+			fake_ArgList[2] = NULL;
+			new_w->end(from, fake_ArgList);
+		}
+		else
+		{
+			snprintf(buffer, 1024, "%s %s", 
+					from, new_w->who_target);
+			if (new_w->who_end)
+			    parse_line(NULL, new_w->who_end, buffer, 0, 0);
+		}
+	} 
+	while (new_w->piggyback && (new_w = new_w->next));
+
+	who_queue_pop();
+	return 1;
+}
+
 
 /*
  *
